/*******************************************************************************
 * Constants used for quick access to the Components object.
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var NewsliveManage = {

  _filled: null,
  get filled() { return this._filled; },
  set filled(val) { this._filled = val; },

  _utils: null,
  get utils() { return this._utils; },
  set utils(val) { this._utils = val; },
  
  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  get listbox() { return document.getElementById("feeds_listbox"); },
    
  // nsISupports
  QueryInterface: function nlm_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },    
  
  // nsIObserver
  observe: function nlm_observe(subject, topic, data) {
    if (this.filled)
      return;
      
    switch (data) {
    
    case "newFeeds":
    case "changedFeeds":
      this.fillListbox();
      this.showMessage("message", false);
      break;

    case "sameFeeds":
      this.showMessage("failed", true);
      break;
        
    default:
      break;    
    }
  },
  
  onLoad: function nlm_onLoad() {
  
    // get the observer service and add ourselves as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                     getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-newslive", false);
           
    // get the utility object
    this.utils = new NewsliveUtils();
    
    // fill the listbox if the service already has items
    if (this.utils.nlService.hasItems("newslive_root", false)) {
      this.fillListbox(); 
      this.showMessage("message", false);
    
    // show the loading message
    } else
      this.showMessage("loading", true);         
  },
  
  onUnload: function nlm_onUnload() {
  
    // remove ourselves as an observer
    this.obService.removeObserver(this, "netscape-newslive");
    
    // clear the variables
    this.filled = null;
    this.utils = null;
    this.obService = null;
  }, 
  
  onSelect: function nlm_onSelect() {
    var listbox = this.listbox;
    
    if (!listbox.selectedItem) {
      this.setButton("up", true);
      this.setButton("down", true);
    
    } else {
      var index = listbox.selectedIndex;
      if (index === 0)
        this.setButton("up", true);
      else
        this.setButton("up", false);
      if (index === listbox.getRowCount()-1)
        this.setButton("down", true);
      else
        this.setButton("down", false);
    }
  },
  
  onAccept: function nlm_onAccept() {
    var listbox = this.listbox;
    var index = listbox.getRowCount();
    var service = this.utils.nlService;
    
    while (index--) {
      var item = listbox.getItemAtIndex(index);
      if (!item)
        continue;
        
      var feed = service.getItem(item.value);
      if (!feed)
        continue;
        
      if (index != feed.sortIndex || item.checked != feed.subscribed) {
        
        if (!item.checked && feed.subscribed)
          service.removeItems(feed.id);      
        feed.sortIndex = index;
        feed.subscribed = item.checked;
        service.updateItem(feed);
      }
    }
    
    window.close();
  },
  
  fillListbox: function nlm_fillListbox() {
    var listbox = this.listbox;
     
    var feeds = this.utils.nlService.getItems("newslive_root", false);
    var index = 0;
    while (index <  feeds.length) {
      var feed = feeds.queryElementAt(index, Ci.nsINewsliveItem);
      
      var item = listbox.appendItem(feed.title, feed.id);
      var checked = (feed.subscribed) ? "true" : "false";
      item.setAttribute("class", "listitem-iconic");
      item.setAttribute("image", feed.iconURL);
      item.setAttribute("type", "checkbox");
      item.setAttribute("checked", checked);
      item.setAttribute("crop", "center");
      
      index++;
    }
    this.filled = true;

    listbox.focus();  
    this.onSelect();
  },
  
  clearListbox: function nlm_clearListbox() {
    var listbox = this.listbox;
    var index = listbox.getRowCount();
    while (index--)
      listbox.removeItemAt(index);
    this.filled = false;

    listbox.focus();  
    this.onSelect();
  },
  
  clearFeeds: function nlm_clearFeeds() {
    var feeds = this.utils.nlService.getItems("newslive_root", false);
    var index = 0;
    while (index < feeds.length) {
      var feed = feeds.queryElementAt(index, Ci.nsINewsliveItem);
      this.utils.nlService.removeItems(feed.id);
      index++;
    }
    this.utils.nlService.removeItems("newslive_root");
    this.utils.nlService.reloadItems(null);
  },
  
  showMessage: function nlm_showMessage(state, disabled) {
    this.setState(state);
    this.setButton("accept", disabled);
    this.setButton("restore", disabled);    
  },
  
  restoreDefaults: function nlm_restoreDefaults() {
    var branch = this.utils.branch;

    var dontAsk = !branch.getBoolPref("showRestore");
    if (!dontAsk) {

      const PREFIX = "newslive.manage.";
      var title = this.utils.getString(PREFIX + "restore.title");
      var message = this.utils.getString(PREFIX + "restore.message");
      var show = this.utils.getString(PREFIX + "restore.show");
    
      var prompt = Cc["@mozilla.org/embedcomp/prompt-service;1"].
                   getService(Ci.nsIPromptService);
      var checkbox = { value: false };
      var open =  prompt.confirmCheck(window, title, message, show, checkbox);
      if (!open)
        return;
        
      branch.setBoolPref("showRestore", !checkbox.value);
    }
    
    this.clearListbox();
    this.showMessage("loading", true);
    this.clearFeeds();
  },
    
  moveUp: function nlm_moveUp() {
    var listbox = this.listbox;
    var item = listbox.selectedItem;
    this.moveItem(item, -1, listbox);
    listbox.focus();  
  },
  
  moveDown: function nlm_moveDown() {
    var listbox = this.listbox;
    var item = listbox.selectedItem;
    this.moveItem(item, 1, listbox);
    listbox.focus();  
  },
  
  moveItem: function nlm_moveItem(item, delta, list) {
    var index = list.getIndexOfItem(item);
    var newIndex = index + delta;
    var max = list.getRowCount() -1;
    
    if (newIndex < 0 || newIndex > max)
      return;
    
    var label = item.label;
    var value = item.value;
    var checked = (item.checked) ? "true" : "false";
    var image = item.image;
    list.removeItemAt(index);
    
    if (newIndex === max)
      item = list.appendItem(label, value);
    else
      item = list.insertItemAt(newIndex, label, value);
    item.setAttribute("class", "listitem-iconic");
    item.setAttribute("image", image);
    item.setAttribute("type", "checkbox");
    item.setAttribute("checked", checked);
    item.setAttribute("crop", "center");
    
    list.ensureElementIsVisible(item);
    list.selectItem(item);  
  },
  
  setButton: function nlm_setButton(prefix, disable) {
    var button = document.getElementById(prefix + "_button");
    if (!disable)
      button.removeAttribute("disabled");
    else
      button.setAttribute("disabled", "true");
  }, 
  
  setState: function nlm_setState(state) {
    var image = document.getElementById("header_icon");
    if (state == "message")
      image.removeAttribute("state");
    else
      image.setAttribute("state", state);
      
    const PREFIX = "newslive.manage.";
    var label = document.getElementById("header_label");
    label.value = this.utils.getString(PREFIX + state);
  }
};  