/*******************************************************************************
 * Quick access to the Components object.
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
/*******************************************************************************
 * Component definitions.
 ******************************************************************************/
const ITEM_CLASSNAME = "Netscape Vote Item";
const ITEM_CLASSID = Components.ID("{3b462c2e-38e3-1e43-8830-98cd7e4cc8f7}");
const ITEM_CONTRACTID = "@netscape.com/vote/item;1";

const SERVICE_CLASSNAME = "Netscape Vote Service";
const SERVICE_CLASSID = Components.ID("{fde7ddab-7a4b-f744-bc3b-00cfc2e4e79a}");
const SERVICE_CONTRACTID = "@netscape.com/vote/service;1";
/*******************************************************************************
 * Contract IDs.
 ******************************************************************************/
const OB_CONTRACTID = "@mozilla.org/observer-service;1";
const PREF_CONTRACTID = "@mozilla.org/preferences-service;1";
const TIMER_CONTRACTID = "@mozilla.org/timer;1";
const REQUEST_CONTRACTID = "@mozilla.org/xmlextras/xmlhttprequest;1";
const MEDIATOR_CONTRACTID = "@mozilla.org/appshell/window-mediator;1";
const WATCHER_CONTRACTID = "@mozilla.org/embedcomp/window-watcher;1";
const BUNDLE_CONTRACTID = "@mozilla.org/intl/stringbundle;1";
const CACHE_CONTRACTID = "@netscape.com/cache/expiring;1";
const CONSOLE_CONTRACTID = "@mozilla.org/consoleservice;1";
const ERROR_CONTRACTID = "@mozilla.org/scripterror;1";
/*******************************************************************************
 * Observer topics
 ******************************************************************************/
const TOPIC_DEFAULT = "netscape-vote";
const TOPIC_SHUTDOWN_APP = "quit-application";
const TOPIC_SHUTDOWN_XPCOM = "xpcom-shutdown";
const TOPIC_PREF = "nsPref:changed";
const TOPIC_TIMER = "timer-callback";
/*******************************************************************************
 * Checker values
 ******************************************************************************/
const CHECKER_URL = "endpoint.api";
const CHECKER_PAUSED = "Server error: " + SERVICE_CLASSNAME + " temporarily disabled for $1 minutes.";
/*******************************************************************************
 * Update Manager values
 ******************************************************************************/
const UPDATE_BUNDLE = "chrome://browser/locale/netscape-vote/vote.properties";
const UPDATE_VOTE = "endpoint.vote";
const UPDATE_SINK = "endpoint.sink";
const UPDATE_TITLE = "vote.module.voted.title";
const UPDATE_MESSAGE = "vote.module.voted.message";
/*******************************************************************************
 * Default values used across components.
 ******************************************************************************/
const DEFAULT_SEC = 1000;                                    // 1 second
const DEFAULT_MIN = 60*DEFAULT_SEC;                          // 1 minute
const DEFAULT_HOUR = 60*DEFAULT_MIN;                         // 1 hour
const DEFAULT_ERROR = "http://www.mozilla.org/newlayout/xml/parsererror.xml";
/*******************************************************************************
 * Service values
 ******************************************************************************/
const SERVICE_BRANCH = "extensions.netscape.vote.";
const SERVICE_DISABLED = "disabled";
const SERVICE_BLACKLIST = "^https?://[^/]*((anchors|design|autos|books|humor|careers|celebrities|diy|donoevil|family|travel|gay|health|personals|men|money|movies|music|news|pets|politics|videos|realestate|religion|science|shopping|sports|tech|tv|videogames|women|www)\.netscape\.com|\.?hotmail\.com|mail\.google\.com)/";
const SERVICE_PERMALINK = "\.netscape\.com/story/[0-9]{4}/[0-9]{2}/[0-9]{2}/.*";
const SERVICE_SCAN = "scan";
const SERVICE_UPDATE = "update";
const SERVICE_EXPIRES = DEFAULT_HOUR;                        // remove items after an hour
const SERVICE_MAX = 100;                                     // hold a maximum of 100 items
/*******************************************************************************
 * Helper object to encode and decode base64 since we cannot use btoa/atop.
 ******************************************************************************/
var Base64 = {

  // private property
  _keyStr: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",
   
  // public method for encoding
  encode: function Base64_encode(input) {
    var output = "";
    var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
    var i = 0;

    input = this._utf8_encode(input);
    while (i<input.length) {

      chr1 = input.charCodeAt(i++);
      chr2 = input.charCodeAt(i++);
      chr3 = input.charCodeAt(i++);

      enc1 = chr1 >> 2;
      enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
      enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
      enc4 = chr3 & 63;

      if (isNaN(chr2))
        enc3 = enc4 = 64;
      else if (isNaN(chr3))
        enc4 = 64;

      output = output +
               this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
               this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);

    }

    return output;
  },

  // public method for decoding
  decode: function Base64_decode(input) {
    var output = "";
    var chr1, chr2, chr3;
    var enc1, enc2, enc3, enc4;
    var i = 0;

    input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");
    while (i<input.length) {

      enc1 = this._keyStr.indexOf(input.charAt(i++));
      enc2 = this._keyStr.indexOf(input.charAt(i++));
      enc3 = this._keyStr.indexOf(input.charAt(i++));
      enc4 = this._keyStr.indexOf(input.charAt(i++));

      chr1 = (enc1 << 2) | (enc2 >> 4);
      chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
      chr3 = ((enc3 & 3) << 6) | enc4;

      output = output + String.fromCharCode(chr1);
      if (enc3 != 64)
        output = output + String.fromCharCode(chr2);  
      if (enc4 != 64)
        output = output + String.fromCharCode(chr3);
    }
    
    output = this._utf8_decode(output);
    return output;
  },

  // private method for UTF-8 encoding
  _utf8_encode: function Base64__utf8_encode(string) {
    string = string.replace(/\r\n/g,"\n");
    var utftext = "";

    for (var n=0; n<string.length; n++) {
      var c = string.charCodeAt(n);
      
      if (c<128)
        utftext += String.fromCharCode(c);
      else if((c>127) && (c<2048)) {
        utftext += String.fromCharCode((c >> 6) | 192);
        utftext += String.fromCharCode((c & 63) | 128);
      } else {
        utftext += String.fromCharCode((c >> 12) | 224);
        utftext += String.fromCharCode(((c >> 6) & 63) | 128);
        utftext += String.fromCharCode((c & 63) | 128);
      }
    }

    return utftext;
  },

  // private method for UTF-8 decoding
  _utf8_decode: function Base64__utf8_decode(utftext) {
    var string = "";
    var i = 0;
    var c = c1 = c2 = 0;

    while (i<utftext.length) {
      c = utftext.charCodeAt(i);

      if (c<128) {
        string += String.fromCharCode(c);
        i++;
      } else if((c>191) && (c<224)) {
        c2 = utftext.charCodeAt(i+1);
        string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
        i += 2;
      } else {
        c2 = utftext.charCodeAt(i+1);
        c3 = utftext.charCodeAt(i+2);
        string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
        i += 3;
      }

    }

    return string;
  }
}; 
/******************************************************************************
 * Get the main application window.  Use this if the type of window does matter.
 * 
 * @return  A window object used for modality.
 *****************************************************************************/
function getMainWindow()
{
  //get the app window
  var mediator = Cc[MEDIATOR_CONTRACTID].getService(Ci.nsIWindowMediator);
  var main = mediator.getMostRecentWindow("navigator:browser");
  if (main)
    return main;
                       
  //open a new window  
  var watcher = Cc[WATCHER_CONTRACTID].getService(Ci.nsIWindowWatcher);                 
  main = watcher.openWindow(null, "chrome://browser/content/browser.xul", 
                            "_blank", "chrome,all,dialog=no", "about:blank");  
  return main;
}
/*******************************************************************************
 * Function to log a message to the error console
 *
 * @param     message        Message to log
 ******************************************************************************/
function logMessage(message) {
  
  // create an error object
  var scriptError = Cc[ERROR_CONTRACTID].createInstance(Ci.nsIScriptError);
  var sourceName = __LOCATION__.path;
  var flag = Ci.nsIScriptError.warningFlag;
  scriptError.init(message, sourceName, "", 0, 0, flag, SERVICE_CONTRACTID);
  
  // log to console
  var consoleService = Cc[CONSOLE_CONTRACTID].getService(Ci.nsIConsoleService);
  consoleService.logMessage(scriptError);
}
/******************************************************************************
 * Get a new prompter.
 *
 * @param   The parent window for the prompter can be null.
 *
 * @return  A new prompter. 
 *****************************************************************************/
function getPrompter(aParent)
{
  //get the watcher service
  var watcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].
                getService(Ci.nsIWindowWatcher);
                
  //return a prompter
  return watcher.getNewPrompter(aParent);
}
/*******************************************************************************
 * Object used to check with the story status api for vote data.
 *
 * @param     service        The vote service.
 ******************************************************************************/
function StatusChecker(service) {
  this.service = service;
  this.load();
}
StatusChecker.prototype = {

  _service: null,
  get service() { return this._service; },
  set service(val) { this._service = val; },
  
  _requests: null,
  get requests() { return this._requests; },
  set requests(val) { this._requests = val; },
   
  _errorCount: null,
  get errorCount() { return this._errorCount; },
  set errorCount(val) { this._errorCount = val; },
   
  _errorTimer: null,
  get errorTimer() { return this._errorTimer; },
  set errorTimer(val) { this._errorTimer = val; },
    
  // nsISupports
  QueryInterface: function sc_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },

  // nsIObserver
  observe: function sc_observe(subject, topic, data) {
  
    // do nothing if not timer topic
    if (topic != TOPIC_TIMER)
      return;
      
    // re-enable after timer
    this.service._paused = false;
  },
  
  load: function sc_load() {
    this.requests = {};
    this.errorCount = 0;
    this.errorTimer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer); 
  },
  
  unload: function sc_unload() {
    
    // stop the timer
    this.errorTimer.cancel();
    
    // stop any requests
    var requests = this.requests;
    for (var URL in requests)
      requests[URL].abort();
    
    // remove variables
    this.requests = null;  
    this.service = null;
    this.errorCount = null;
    this.errorTimer = null;
  },

  addURL: function sc_addURL(URL) {
      
    // do nothing if already requesting
    if (this.requests.hasOwnProperty(URL))
      return;
      
    // request update
    this.requestUpdate(URL);    
  },
  
  requestUpdate: function sc_requestUpdate(URL) {
           
    // create an XML HTTP request  
    var self = this;
    var request = Cc[REQUEST_CONTRACTID].createInstance(Ci.nsIXMLHttpRequest);
    var api = this.service._branch.getCharPref(CHECKER_URL); 
    api += Base64.encode(URL);
    request.open("GET", api, true);
    request.overrideMimeType("text/xml");
    request.setRequestHeader("X-Netscape", "Navigator");        
    request.QueryInterface(Ci.nsIJSXMLHttpRequest).onerror = function() { self.onError(URL); };
    request.QueryInterface(Ci.nsIJSXMLHttpRequest).onload = function() { self.onLoad(URL); };
    this.requests[URL] = request;    
       
    // send the request
    this.requests[URL].send(null);  
  },
  
  onLoad: function sc_onLoad(URL) {
      
    // get the vote item from the xml
    var request = this.requests[URL];
    var item = null;
    try {
      item = this.createItem(request.responseXML);
    } catch(e) {}
    
    // item could not be created
    if (!item) {
      this.onError(URL);
      return;
    }
    
    // remove the request
    delete this.requests[URL]; 

    // reset the error counter
    this.errorCount = 0;
        
    // cache the data and notify the front-end
    this.service.setItem(item, true);
  },
  
  onError: function sc_onError(URL) {
  
    // remove the request
    delete this.requests[URL];
    
    // log to the console
    this.errorCount++;
    var delay = Math.pow(this.errorCount, 2);
    var message = CHECKER_PAUSED.replace("$1", delay);
    logMessage(message);
        
    // increment the error counter and start the disabled timer   
    this.service._paused = true;
    this.errorTimer.init(this, delay*DEFAULT_MIN, Ci.nsITimer.TYPE_ONE_SHOT);    
  },
  
  createItem: function sc_createItem(xml) {
  
    // parse error
    if (!xml || xml.documentElement.namespaceURI == DEFAULT_ERROR)
      return null;
         
    // parse the vote data
    var data = {  URL: "",
                  submitted: false,
                  voted: false,
                  voteCount: 0,
                  sinkCount: 0,
                  commentCount: 0,
                  permaLink: ""
                };
    data.URL = xml.getElementsByTagName("story")[0].getElementsByTagName("url")[0].firstChild.nodeValue;
    data.submitted = (xml.getElementsByTagName("story")[0].getAttribute("submitted") == "true") ? true : false;
    data.voted = (xml.getElementsByTagName("story")[0].getAttribute("votedOn") == "true") ? true : false;
    
    if (data.submitted){
      data.voteCount = parseInt(xml.getElementsByTagName("votecount")[0].firstChild.nodeValue);
      data.sinkCount = parseInt(xml.getElementsByTagName("sinkcount")[0].firstChild.nodeValue);
      data.commentCount = parseInt(xml.getElementsByTagName("commentcount")[0].firstChild.nodeValue);
      data.permaLink = xml.getElementsByTagName("link")[0].firstChild.nodeValue;
    }
    
    // create a vote item and return it
    var item = new VoteItem();
    item.init(data.URL, data.submitted, data.voted, data.voteCount, 
              data.sinkCount, data.commentCount, data.permaLink);
    return item;                         
  }  
};
/*******************************************************************************
 * Object used to update the vote status.
 *
 * @param     service        The vote service.
 ******************************************************************************/
function UpdateManager(service) {
  this.service = service;
  this.load();
}
UpdateManager.prototype = {
  
  _service: null,
  get service() { return this._service; },
  set service(val) { this._service = val; },
  
  _requests: null,
  get requests() { return this._requests; },
  set requests(val) { this._requests = val; },
    
  // nsISupports
  QueryInterface: function sc_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },

  load: function upm_load() {
    this.requests = {};
  },
  
  unload: function upm_unload() {
    
    // stop any requests
    var requests = this.requests;
    for (var URL in requests)
      requests[URL].abort();

    // remove variables
    this.requests = null;  
    this.service = null;
  },
  
  addUpdate: function upm_addUpdate(item, type) {
    if (this.requests.hasOwnProperty(item.URL))
      return;
    this.requestUpdate(item, type);    
  },
  
  requestUpdate: function upm_requestUpdate(item, type) {
           
    // create an XML HTTP request  
    var self = this;
    var URL = item.URL;
    var request = Cc[REQUEST_CONTRACTID].createInstance(Ci.nsIXMLHttpRequest);
    var api = null;
    if (type == Ci.nsIVoteService.TYPE_VOTE)
      api = this.service._branch.getCharPref(UPDATE_VOTE);
    else
      api = this.service._branch.getCharPref(UPDATE_SINK); 
    api += Base64.encode(URL);
    request.open("GET", api, true);
    request.overrideMimeType("text/xml");
    request.setRequestHeader("X-Netscape", "Navigator");        
    request.onerror = function() { self.onError(item); };
    request.onload = function() { self.onLoad(item); };
    this.requests[URL] = request;    
    
    // send the request
    this.requests[URL].send(null);  
  },
  
  onLoad: function upm_onLoad(item) {
  
    // get the vote item from the xml
    var request = this.requests[item.URL];
    try {
      this.updateItem(request.responseXML, item);
    } catch(e) {}
    
    // remove the request
    delete this.requests[item.URL]; 
  },
  
  onError: function upm_onError(item) {
  
    // remove the item
    this.service.removeItem(item);
    
    // remove the request
    delete this.requests[item.URL];     
  },
  
  updateItem: function upm_updateItem(xml, item) {
  
    // parse error
    if (!xml || xml.documentElement.namespaceURI == DEFAULT_ERROR) {
      this.service.removeItem(item);    
      return;
    }
       
    // get the vote root
    var voteRoot = xml.getElementsByTagName("vote")[0];    
    
    // check for loggedIn
    var loggedIn = voteRoot.getAttribute("loggedIn");
    if (loggedIn != "1") {
    
      // remove the vote data
      this.service.removeItem(item);

      // redirect to sign in
      var redir = voteRoot.getElementsByTagName("link")[0].childNodes[0].nodeValue;
      var win = getMainWindow();
      win.Vote.openLink(null, redir);
      return;
    }
    
    // parse the vote data
    var data = {  URL: item.URL,
                  submitted: item.submitted,
                  voted: true,
                  voteCount: item.voteCount,
                  sinkCount: item.sinkCount,
                  commentCount: item.commentCount,
                  permaLink: item.permaLink
                };                  
    data.voteCount = parseInt(voteRoot.getElementsByTagName("votecount")[0].firstChild.nodeValue);
    data.sinkCount = parseInt(voteRoot.getElementsByTagName("sinkcount")[0].firstChild.nodeValue);    
    item.init(data.URL, data.submitted, data.voted, data.voteCount, 
              data.sinkCount, data.commentCount, data.permaLink);
      
    // cache the data and notify the front-end
    this.service.setItem(item, true);
                  
    // show dialog if already voted
    var showAlert = (voteRoot.getAttribute("counted") == "0") ? true : false;
    if (showAlert)
      this.showPrompt();                      
  },
  
  showPrompt: function upm_showPrompt() {
  
    // setup string bundle
    var sbService = Cc[BUNDLE_CONTRACTID].getService(Ci.nsIStringBundleService);
    bundle = sbService.createBundle(UPDATE_BUNDLE);     
    
    // get strings
    var title = bundle.GetStringFromName(UPDATE_TITLE);
    var message = bundle.GetStringFromName(UPDATE_MESSAGE);
    
    // show prompt
    var prompter = getPrompter(getMainWindow());
        prompter.alert(title, message);    
  }   
};
/*******************************************************************************
 * Interface used to describe a vote item.  This interface is used to 
 * represent meta data describing a story's vote status on netscape.com
 * 
 * @version   1.0
 ******************************************************************************/
function VoteItem() {}
VoteItem.prototype = {
  _URL: null,
  _submitted: null,
  _voted: null,
  _voteCount: null,
  _sinkCount: null,
  _commentCount: null,
  _permaLink: null,
  
  // nsISupports
  QueryInterface: function vti_QI(iid) {
    if (iid.equals(Ci.nsIVoteItem) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },

  // nsIVoteItem
  get URL() { return this._URL; },
  get submitted() { return this._submitted; },
  get voted() { return this._voted; },
  get voteCount() { return this._voteCount; },
  get sinkCount() { return this._sinkCount; },
  get commentCount() { return this._commentCount; },
  get permaLink() { return this._permaLink; },
  
  init: function vti_init(URL, submitted, voted, voteCount, 
                          sinkCount, commentCount, permaLink) {

    this._URL = URL;
    this._submitted = submitted;
    this._voted = voted;
    this._voteCount = voteCount;
    this._sinkCount = sinkCount;
    this._commentCount = commentCount;
    this._permaLink = permaLink;
  },

  clone: function vti_clone() {
    var rv = new VoteItem();
    
    rv.init(this._URL, this._submitted, this._voted, this._voteCount, 
            this._sinkCount, this._commentCount, this._permaLink);
    
    return rv;
  }     
};
/*******************************************************************************
 * The main vote interface.  This interface takes care of updating the 
 * vote status data.  It notifies the front-end
 * when updates take place.
 * 
 * @version   1.0
 ******************************************************************************/
function VoteService() {
  this._load();
}
VoteService.prototype = {
  _obService: null,
  _branch: null,
  _disabled: null,
  _paused: null,
  _blackList: null,
  _permaLink: null, 
  _checker: null, 
  _updater: null, 
  _cache: null,
          
  // nsISupports
  QueryInterface: function vts_QI(iid) {
    if (iid.equals(Ci.nsIVoteService) ||
        iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIObserver
  observe: function vts_observe(subject, topic, data) {
    switch (topic) {
    
    case TOPIC_SHUTDOWN_APP:
      this._unload();
      break;
              
    case TOPIC_SHUTDOWN_XPCOM:
      this._unloadFinal();
      break;
    
    case TOPIC_PREF:
      if (data == SERVICE_DISABLED)
        this._disabled = this._branch.getBoolPref(SERVICE_DISABLED);
      break;
      
    default:
      return;
    }
  },
  
  // nsIVoteService
  get disabled() { return this._disabled; },
  get paused() { return this._paused; },
  
  checkURL: function vts_checkURL(URL) {
    
    // return null if disabled or paused
    if (this.disabled || this.paused)
      return null;
      
    // return null if not an http or ftp site
    if (!URL || (URL.indexOf("http") !== 0 && URL.indexOf("ftp") !== 0))
      return null;
    
    // match on the domain blacklist
	if (URL.match(this._blackList)) {
  
      // match on permalink
      if (URL.match(this._permaLink)) {
        
        // create a vote item and notify the front-end we need a document scan  
        var item = new VoteItem();
        item.init("", true, false, 0, 0, 0, URL);
        this._notify(item, SERVICE_SCAN);
        return null;
      }
      
      // blacklist and not a permalink
      return null;
    }
    
    // return the stored url
    if (this._cache.hasKey(URL))
      return this._cache.getValue(URL);
  
    // return null and submit a check request
    this._checker.addURL(URL);
    return null;
  },
  
  scanDocument: function vts_scanDocument(doc) {
    
    // do nothing if disabled or paused
    if (this.disabled || this.paused)
      return;
      
    // loop through the link elements        
    var links = doc.getElementsByTagName("link");            
    for (var i=0; i<links.length; i++) {
    
      // release attribute not equal to bookmark
      var link = links[i];
      if (link.getAttribute("rel") != "bookmark")
        continue;
        
      // title attribute not equal to Source
      if (link.getAttribute("title") != "Source")
        continue;
          

      // store the link data              
      var data = {  URL: "",
                    submitted: false,
                    voted: false,
                    voteCount: 0,
                    sinkCount: 0,
                    commentCount: 0,
                    permaLink: ""
                  };    
      data.submitted = true;
      data.URL = link.getAttribute("href");
      data.permaLink = doc.location.href;
      
      // fixup the permalink
      var idx = data.permaLink.indexOf("#");
      if (idx != -1)
        data.permaLink = data.permaLink.substr(0, idx);
                      
      // find the vote and comment values.
      var metas = doc.getElementsByTagName("meta");            
      for (var x=0; x<metas.length; x++) {
      
        var meta = metas[x];
        var name = meta.getAttribute("name");
        var content = meta.getAttribute("content");
        switch  (name) {
        
        case "votes":
          data.voteCount = parseInt(content);
          break;
          
        case "sinks":
          data.sinkCount = parseInt(content);
          break;
          
        case "comments":
          data.commentCount = parseInt(content);
          break;
          
        default:
          break;
        }
      }
      
      // store the value
      var item = new VoteItem();
      item.init(data.URL, data.submitted, data.voted, data.voteCount, 
                data.sinkCount, data.commentCount, data.permaLink);
      this.setItem(item, false);
      break;
    }  
  },
  
  submitUpdate: function vts_submitUpdate(item, type) {
    this._updater.addUpdate(item.clone(), type);
  },
  
  getItem: function vts_getItem(URL) {
    return this._cache.getValue(URL);
  },
  
  setItem: function vts_setItem(item, notify) {
    this._cache.setValue(item.URL, item.clone());
    if (notify)
      this._notify(item, SERVICE_UPDATE); 
  },
  
  removeItem: function vts_removeItem(item) {
    this._cache.deleteValue(item.URL);
  },
        
  _load: function vts__load() {
  
    // add observers
    this._obService = Cc[OB_CONTRACTID].getService(Ci.nsIObserverService);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_APP, false);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_XPCOM, false);
    this._branch = Cc[PREF_CONTRACTID].getService(Ci.nsIPrefService).
                   getBranch(SERVICE_BRANCH);
    this._branch.QueryInterface(Ci.nsIPrefBranch2).addObserver("", this, false);
        
    // setup variables
    this._disabled = this._branch.getBoolPref(SERVICE_DISABLED);   
    this._paused = false; 
    this._blackList = new RegExp(SERVICE_BLACKLIST, "i");
    this._permaLink = new RegExp(SERVICE_PERMALINK, "i");
    this._checker = new StatusChecker(this);
    this._updater = new UpdateManager(this);
    this._cache = Cc[CACHE_CONTRACTID].createInstance(Ci.nsIExpiringCache);       
    this._cache.init(SERVICE_MAX, SERVICE_EXPIRES, false);
  },
  
  _unload: function vts__unload() {
    this._checker.unload();
    this._updater.unload();
    this._cache.destroy();
  },
  
  _unloadFinal: function vts__unloadFinal() {
         
    // remove observers
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_APP);    
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_XPCOM);
    this._branch.QueryInterface(Ci.nsIPrefBranch2).removeObserver("", this);
        
    // remove variables
    this._obService = null;
    this._branch = null;
    this._disabled = null;
    this._paused = null;
    this._blackList = null;
    this._permaLink = null;
    this._checker = null;  
    this._updater = null;  
    this._cache = null;
  },
  
  _notify: function vts__notifyObservers(subject, data) {
    try {
      this._obService.notifyObservers(subject, TOPIC_DEFAULT, data);
    } catch(e) {
      Components.utils.reportError(e);
    }
  }     
};
/*******************************************************************************
 * Generic object used as a factory based on a passed in constructor.
 ******************************************************************************/
function GenericFactory(ctor) {
  this._ctor = ctor;
}  
GenericFactory.prototype = {
  _ctor: null,
  
  // nsISupports
  QueryInterface: function gf_QI(iid) {
    if (iid.equals(Ci.nsIFactory) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIFactory
  createInstance: function gf_createInstance(outer, iid) {
    if (outer !== null)
      throw Cr.NS_ERROR_NO_AGGREGATION;
    return (new this._ctor()).QueryInterface(iid);
  },

  lockFactory: function gf_lockFactory(loc) {
    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
  }
};
/*******************************************************************************
 * Object implementing the nsIModule interface.  This is used to retrieve 
 * the different components and services the module implements.
 ******************************************************************************/
var Module = {

  // nsISupports
  QueryInterface: function mod_QI(iid) {
    if (iid.equals(Ci.nsIModule) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE;
  },
  
  // nsIModule
  getClassObject: function mod_getClassObject(cm, cid, iid) {
    if (!iid.equals(Ci.nsIFactory))
      throw Cr.NS_ERROR_NOT_IMPLEMENTED;
    
    if (cid.equals(ITEM_CLASSID))
      return new GenericFactory(VoteItem);
    else if (cid.equals(SERVICE_CLASSID))
      return new GenericFactory(VoteService);
          
    throw Cr.NS_ERROR_NO_INTERFACE;
  },

  registerSelf: function mod_registerSelf(cm, file, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);  
    
    cr.registerFactoryLocation(ITEM_CLASSID, ITEM_CLASSNAME, ITEM_CONTRACTID,
                               file, location, type);
    cr.registerFactoryLocation(SERVICE_CLASSID, SERVICE_CLASSNAME, SERVICE_CONTRACTID,
                               file, location, type);                                                                                           
  },

  unregisterSelf: function mod_unregisterSelf(cm, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);
    
    cr.unregisterFactoryLocation(ITEM_CLASSID, location);
    cr.unregisterFactoryLocation(SERVICE_CLASSID, location);
  },

  canUnload: function mod_canUnload(cm) { return true; }
};
/*******************************************************************************
 * Entry point used to retrieve the module object.
 ******************************************************************************/
function NSGetModule(cm, file) { return Module; }